/*============================================================================
 * Interpolation by cubic splines in n dimensions.
 * Works with data located w.r.t. regular grids possibly stretched.
 * It accepts extrapolation but the user should be aware that extrapolating
 * from 3rd order polynomials can quickly lead to absurd values. 
 * The functions cub_interpolate_1 & cub_interpolate_n are from: 
 * http://www.paulinternet.nl/?page=bicubic (november 2015)
 * It is also possible to select a simpler linear interpolation.
 *============================================================================*/

/* Code_Saturne version 4.2-alpha */

/*
  This file is part of Code_Saturne, a general-purpose CFD tool.

  Copyright (C) 1998-2015 EDF S.A.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
  Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
 * Standard C library headers
 *----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
 * Local headers
 *----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/*!
 * \brief  Linear interpolation 1-D
 */
/*----------------------------------------------------------------------------*/

double lin_interpolate_1 (double pi[2], 
			  double xi);

/*----------------------------------------------------------------------------*/
/*!
 * \brief  Linear interpolation N-D
 */
/*----------------------------------------------------------------------------*/

double lin_interpolate_n (int     ndim, 
			  double *pi,
			  double *xi);

/*----------------------------------------------------------------------------*/
/*!
 * \brief  Cubic interpolation 1-D
 */
/*----------------------------------------------------------------------------*/

double cub_interpolate_1 (double pi[4], 
			  double xi);

/*----------------------------------------------------------------------------*/
/*!
 * \brief  Cubic interpolation N-D
 */
/*----------------------------------------------------------------------------*/

double cub_interpolate_n (int     ndim, 
			  double *pi, 
			  double *xi);

/*----------------------------------------------------------------------------*/
/*!
 * \brief  Find the rank in each dimension from the global index
 */
/*----------------------------------------------------------------------------*/

int* find_indices(int  ndim, 
		  int  ix, 
		  int *nx);

/*----------------------------------------------------------------------------*/
/*!
 * \brief  Prepare data for interpolation 
 *
 * \param [in]  cubint : interpolation with cubic splines if set to 1, linear otherwise
 * \param [in]  dernull : first derivatives null at boudaries if set to 1, second derivatives
 *     null otherwise
 * \param [in]  stretched : if the grid made by the locations of the data to be interpolated is
 *     stretched, this parameter should be set to 1, otherwise set to 0, a step can 
 *     be avoided in this function
 * \param [in]  nd : number of physical dimensions of the problem (between 1 and infty)
 * \param [in]  ng : number of points in the data to be interpolated IN EACH DIMENSION
 * \param [in]  xglob : locations of the points aforementioned. This has to be carefully organized.
 *     Considering ntot the total number of points 
 *     - in 2d (xi,yi)    -> (x(i+ntot), x(i))
 *     - in 3d (xi,yi,zi) -> (x(i+2*ntot), x(i+ntot), x(i)).
 *     Also the locations are assumed to be sorted from min to max or max to min.
 * \param [in]  pglob : values at the points aforementioned
 * \param [in]  ni : number of points where interpolated values have to be calculated
 * \param [in]  xintp : locations of the points aforementioned
 * \param [out] pintp : values at the points aforementioned
 */
/*----------------------------------------------------------------------------*/

void cs_cub_interpolate(int     cubint, 
			int     dernull, 
			int     stretched, 
			int     ndim,
			int    *ng, 
			double *xglob, 
			double *pglob, 
			int     sizepi, 
			double *xintp, 
			double *pintp);
